/*******************************************************************************
 * Copyright (c) 2016, 2018 fortiss GmbH.
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v2.0 which is available at
 * http://www.eclipse.org/legal/epl-v20.html
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Florian Hoelzl (fortiss GmbH) - initial implementation
 *     Alexander Diewald (fortiss GmbH) - backport to AF3 & Integration
 *                                        improvements
 *
 *******************************************************************************/
package org.fortiss.tooling.common.ui.javafx.control.treetableview;

import javafx.beans.property.SimpleObjectProperty;
import javafx.scene.Parent;
import javafx.scene.control.SelectionMode;
import javafx.scene.control.TextField;
import javafx.scene.control.TreeItem;
import javafx.scene.control.TreeTableColumn;
import javafx.scene.control.TreeTableView;
import javafx.scene.control.TreeTableView.TreeTableViewSelectionModel;
import javafx.scene.layout.BorderPane;

/**
 * A JavaFX {@link TreeTableView} based on an {@link DynamicTreeContentProviderBase} and
 * {@link DynamicTreeTableUIProviderBase}. This class is intended to be used as a wrapper for
 * {@link TreeTableView} and its rather complex expert API.
 * <P>
 * This class currently supports only a static tree-table layout, i.e., there is no way to alter the
 * columns unless the developer changes the underlying {@link TreeTableView} directly. Note that if
 * a column is removed the index provided to the label provider might not be correct anymore.
 * <P>
 * Furthermore, the selection model is currently fixed to be row selection.
 */
public final class DynamicTreeTableViewer<T> extends DynamicTreeViewerBase<T> {
	/** The {@link TreeTableView} control to be managed. */
	private final TreeTableView<T> view;
	/** The filter text field if filtering is enabled in content provider. */
	private final TextField filter;
	/** The pane for filter and tree widgets. */
	private final BorderPane pane;
	/** The content provider of this tree-table. */
	private final DynamicTreeContentProviderBase<T> contentProvider;
	/** The UI provider of this tree-table. */
	private final DynamicTreeTableUIProviderBase<T> uiProvider;

	/** Constructor. */
	public DynamicTreeTableViewer(TreeTableView<T> view, T root, boolean showRoot, int revealLevel,
			DynamicTreeContentProviderBase<T> contentProvider,
			DynamicTreeTableUIProviderBase<T> uiProvider) {
		super(contentProvider);
		this.contentProvider = contentProvider;
		this.uiProvider = uiProvider;
		// construct view
		this.view = view;
		if(contentProvider.enableFiltering()) {
			filter = new TextField();
			filter.textProperty().addListener((obs, oVal, nVal) -> {
				update();
			});
			pane = new BorderPane();
			pane.setTop(filter);
			pane.setCenter(view);
		} else {
			filter = null;
			pane = null;
		}
		DynamicTreeItem<T> rootItem = new DynamicTreeItem<T>(root, this);
		view.setRoot(rootItem);
		view.setShowRoot(showRoot);
		view.setEditable(true);
		TreeTableViewSelectionModel<T> selectionModel = view.getSelectionModel();
		selectionModel.setSelectionMode(SelectionMode.MULTIPLE);
		selectionModel.selectedItemProperty().addListener((obs, oVal, nVal) -> {
			T ov = (oVal != null) ? oVal.getValue() : null;
			T nv = (nVal != null) ? nVal.getValue() : null;
			uiProvider.select(ov, nv);
		});

		rootItem.update();
		// expand to reveal (+1 if root node is not shown
		expandItem(rootItem, showRoot ? revealLevel : revealLevel + 1);
	}

	/** Constructor. */
	public DynamicTreeTableViewer(T root, boolean showRoot, int revealLevel,
			DynamicTreeContentProviderBase<T> contentProvider,
			DynamicTreeTableUIProviderBase<T> uiProvider) {
		this(new TreeTableView<T>(), root, showRoot, revealLevel, contentProvider, uiProvider);
	}

	/** Updates the viewer content. */
	public void update() {
		// wild cast works: see constructor
		DynamicTreeItem<T> rootItem = (DynamicTreeItem<T>)view.getRoot();
		rootItem.update(filter != null ? filter.getText() : null);
	}

	/** Expands items up to the given level. */
	private void expandItem(TreeItem<T> parentItem, int revealLevel) {
		if(revealLevel <= 0) {
			return;
		}
		parentItem.setExpanded(true);
		revealLevel--;
		for(TreeItem<T> child : parentItem.getChildren()) {
			expandItem(child, revealLevel);
		}
	}

	/**
	 * Adds a column to the table part of the view. The labels, context menus and icons are shown as
	 * defined in the {@link DynamicTreeTableUIProviderBase}.
	 */
	public TreeTableColumn<T, String> addColumn(String headerLabel, int prefWidth) {
		int num = view.getColumns().size();
		TreeTableColumn<T, String> column = new TreeTableColumn<>(headerLabel);

		column.setPrefWidth(prefWidth);
		column.setCellValueFactory(param -> {
			T data = param.getValue().getValue();
			return new SimpleObjectProperty<String>(uiProvider.getLabel(data, num));
		});

		uiProvider.applyToColumn(num, column);
		view.getColumns().add(column);
		return column;
	}

	/**
	 * Adds a column to the table part of the view. The labels, context menus and icons are shown as
	 * defined in the {@link DynamicTreeTableUIProviderBase}.
	 * 
	 * @deprecated use {@link DynamicTreeTableUIProviderBase}
	 */
	@Deprecated
	public TreeTableColumn<T, String> addColumn(String headerLabel, int prefWidth,
			@SuppressWarnings("unused") boolean readOnly) {
		return addColumn(headerLabel, prefWidth);
	}

	/** Returns the underlying {@link TreeTableView}. */
	public TreeTableView<T> getControl() {
		return view;
	}

	/** Returns the filter pane or the tree view if filtering is disabled. */
	public Parent getFilterControl() {
		if(contentProvider.enableFiltering()) {
			return pane;
		}
		return getControl();
	}
}
